package widebase.stream.codec

import org.jboss.netty.buffer.ChannelBuffer
import org.jboss.netty.channel.Channel
import org.jboss.netty.channel.ChannelHandlerContext

import org.jboss.netty.handler.codec.frame. {

  CorruptedFrameException,
  LengthFieldBasedFrameDecoder

}

import vario.data.sizeOf

/** Decode requests.
 *
 * @author myst3r10n
 */
class RequestDecoder
  extends LengthFieldBasedFrameDecoder(
    Int.MaxValue,
    0,
    sizeOf.int,
    0,
    sizeOf.int)
  with MessageTypeDecoder {

  import MessageType.MessageType

  @throws(classOf[Exception])
  override def decode(
    ctx: ChannelHandlerContext,
    channel: Channel,
    buffer: ChannelBuffer): Object = {

    val frame = strip(ctx, channel, buffer)

    if(frame == null)
      return null

    val id = readId(frame)

    if(id.isEmpty)
      return null

    readMessage(id.get, frame)

  }

  protected def readMessage(id: Int, buffer: ChannelBuffer): Object = {

    val messageType =
      try {

        MessageType(id)

      } catch {

        case e: NoSuchElementException =>
          buffer.resetReaderIndex
          throw new CorruptedFrameException("Unfamiliar with message id: " + id)

      }

    messageType match {

      case MessageType.LoginMessage =>
        val username = readString(buffer)

        if(username == null) {

          buffer.resetReaderIndex
          null

        }

        val password = readString(buffer)

        if(password == null) {

          buffer.resetReaderIndex
          null

        }

        new LoginMessage(username, password)

      case MessageType.RemoteShutdownMessage => new RemoteShutdownMessage
      case _ =>
        buffer.resetReaderIndex
        null

    }
  }

  protected def strip(
    ctx: ChannelHandlerContext,
    channel: Channel,
    buffer: ChannelBuffer) =
    super.decode(ctx, channel, buffer).asInstanceOf[ChannelBuffer]

}

