package widebase.stream.codec

import org.jboss.netty.buffer.ChannelBuffer
import org.jboss.netty.channel.Channel
import org.jboss.netty.channel.ChannelHandlerContext

import org.jboss.netty.handler.codec.frame. {

  CorruptedFrameException,
  LengthFieldBasedFrameDecoder

}

import vario.data.sizeOf

/** Decode response.
 *
 * @author myst3r10n
 */
class ResponseDecoder
  extends LengthFieldBasedFrameDecoder(
    Int.MaxValue,
    0,
    sizeOf.int,
    0,
    sizeOf.int)
  with MessageTypeDecoder {

  import MessageType.MessageType

  @throws(classOf[Exception])
  override def decode(
    ctx: ChannelHandlerContext,
    channel: Channel,
    buffer: ChannelBuffer): Object = {

    val frame = strip(ctx, channel, buffer)

    if(frame == null)
      return null

    val id = readId(frame)

    if(id.isEmpty)
      return null

    readMessage(id.get, frame)

  }

  @throws(classOf[CorruptedFrameException])
  protected def readMessage(id: Int, buffer: ChannelBuffer): Object = {

    val messageType =
      try {

        MessageType(id)

      } catch {

        case e: NoSuchElementException =>
          buffer.resetReaderIndex
          throw new CorruptedFrameException("Unfamiliar with message id: " + id)

      }

    messageType match {

      case MessageType.BadMessage => new BadMessage
      case MessageType.DoneMessage => new DoneMessage
      case MessageType.ForbiddenMessage => new ForbiddenMessage
      case MessageType.LoginFailedMessage => new LoginFailedMessage
      case MessageType.LoginGrantedMessage => new LoginGrantedMessage
      case MessageType.LoginRequiredMessage => new LoginRequiredMessage
      case MessageType.UnauthorizedMessage => new UnauthorizedMessage
      case _ =>
        buffer.resetReaderIndex
        null

    }
  }

  protected def strip(
    ctx: ChannelHandlerContext,
    channel: Channel,
    buffer: ChannelBuffer) =
    super.decode(ctx, channel, buffer).asInstanceOf[ChannelBuffer]

}

