package widebase.stream.codec

import org.jboss.netty.buffer. { ChannelBuffer, ChannelBuffers }
import org.jboss.netty.channel.Channel
import org.jboss.netty.channel.ChannelHandlerContext
import org.jboss.netty.handler.codec.oneone.OneToOneEncoder

/** Encode requests.
 *
 * @author myst3r10n
 */
class ResponseEncoder extends OneToOneEncoder with MessageTypeEncoder {

  @throws(classOf[Exception])
  override def encode(
    ctx: ChannelHandlerContext,
    channel: Channel,
    msg: Object): Object = writeMessage(ChannelBuffers.dynamicBuffer, msg)

  protected def writeMessage(buffer: ChannelBuffer, message: Object): Object = {

    message match {

      case message: BadMessage => writeId(buffer, MessageType.BadMessage.id)
      case message: DoneMessage => writeId(buffer, MessageType.DoneMessage.id)
      case message: ForbiddenMessage => writeId(buffer, MessageType.ForbiddenMessage.id)
      case message: LoginFailedMessage => writeId(buffer, MessageType.LoginFailedMessage.id)
      case message: LoginGrantedMessage => writeId(buffer, MessageType.LoginGrantedMessage.id)
      case message: LoginRequiredMessage => writeId(buffer, MessageType.LoginRequiredMessage.id)
      case message: UnauthorizedMessage => writeId(buffer, MessageType.UnauthorizedMessage.id)
      case message => return message

    }

    finish(buffer)
    
    buffer

  }
}

